/*
 * Written by TeTeT
 * basic flight operations: Create a plane on elevator 3, taxi to cat 3, fly a circle, land, taxi to ele 2, repeat
 */


#include "Config.h"
#include "Common.h"

#define DISTANCE_TOW 11
#define SPEEDTAXI 3
#define DELAY 10
#define TANKERALT 2000
#define FLIGHTALT 1000

private ["_nimspots", "_posCat", "_posE2", "_posE3", "_posE4", "_posWires", "_doFlightOps",
		 "_ele2Obj", "_ele3Obj", "_ele4Obj", "_jets", "_jetOne", "_jetTwo", "_plane", "_pos",
		 "_approachPos", "_approach", "_grp", "_createTractor", "_driveTo", "_takeOffPos", "_dir",
		 "_carrier7", "_posFuel1", "_posFuel2", "_posEle2Tow1", "_pos2Ele2Tow2", "_posEle2PushBack",
		 "_parkingTractor", "_posEle2Approach", "_tractor", "_posTankerLoop1", "_posTankerLoop2",
		 "_appSpeed", "_aligned"];

params [ ["_dir", 0], ["_nim", ObjNull] ];

"Flight operations initiated" call BIS_fnc_log;

_doFlightOps = ["doFlightOps", _nim] call TTT_fnc_global;

if (!_doFlightOps) exitWith {};

// check requirements
if (!isClass(configFile >> "CfgPatches" >> "JS_JC_FA18")) exitWith { systemChat "Need F/A-18 by John_Spartan and Saul for flight operations." };
if (!isClass(configFile >> "CfgPatches" >> "lesh_towing")) exitWith { systemChat "Need towing by Leshrack for flight operations." };
if (!isClass(configFile >> "CfgPatches" >> "Peral_Airfield_Logistics")) exitWith { systemChat "Need airfield logistics by Peral for flight operations." };


// initialize various positions on the carrier
_nimspots = ["nimspots", _nim] call TTT_fnc_global;
_posCat = _nimspots modeltoworld (_nimspots selectionposition "C3");
_posE2 = _nimspots modeltoworld (_nimspots selectionposition "E2");
_posE3 = _nimspots modeltoworld (_nimspots selectionposition "E3");
_posE4 = _nimspots modeltoworld (_nimspots selectionposition "E4");
_posWires = _nimspots modeltoworld (_nimspots selectionposition "WireCenter");

_carrier7 = ["JDG_carrier_deck_7", _nim] call TTT_fnc_global;
_posFuel1 = _carrier7 modelToWorld (_carrier7 selectionPosition "fuelHose1pos");
_posFuel2 = _carrier7 modelToWorld (_carrier7 selectionPosition "fuelHose2pos");

_dir = getDir _nimspots;
_posEle2Approach = CIRCLEPOS2(_posFuel1, _dir - 90, 5);
_posEle2Approach pushBack DECK;
_posEle2Tow1 = CIRCLEPOS2(_posE2, _dir - 180, 10);
_posEle2Tow1 pushBack DECK;
_posEle2Tow2 = CIRCLEPOS2(_posE2, _dir - 90, 15);
_posEle2Tow2 pushBack DECK;
_posEle2PushBack = CIRCLEPOS2(_posE2, _dir - 90, 8);
_posEle2PushBack pushBack DECK;
_parkingTractor = CIRCLEPOS2(_posWires, _dir + 90, 20);
_parkingTractor pushBack DECK;

// find elevator objects, ele1 is blocked by parked plane in ambiance
_ele2Obj = _posE2 nearestObject "JDG_carrier_ele_2";
_ele3Obj = _posE3 nearestObject "JDG_carrier_ele_3";
_ele4Obj = _posE4 nearestObject "JDG_carrier_ele_4";

_jets = [] call TTT_fnc_planeTypes;
_jetOne = _jets select 0;
_jetTwo = _jets select 1;

// Some preconditions for this do exist: the plane flies in basically
// a straight line towards _approach
// The plan is to find a spot besides _approach, fly there and then do a 180 degree turn
// so we're aligned with _nim at _approach
_turnAndAlign = {
	private ["_appSpeed", "_turn", "_aligned", "_turnIn", "_pos", "_dir"];
	params ["_plane", "_approach", "_nim", "_v"];

	// kill the crew
	/*{_x setDamage 1} forEach crew _plane;*/

	_aligned = false;
	_appSpeed = 400 / 3.6;
	_turn = "none";

	_pos = getPos _approach;
	_dir = ([_approach, _nim] call BIS_fnc_dirTo) + 90;
	_turnIn = CIRCLEPOS2(_pos, _dir, 500);
	_turnIn pushBack (getPosASL _approach select 2);

	systemChat "Flying to turnIn";
	// first we fly to turnIn, then we turn
	while {_plane distance _turnIn > 100} do {
		_v = [getPos _plane, _turnIn] call BIS_fnc_vectorFromXToY;
		_v = [ (_v select 0) * _appSpeed,
		         (_v select 1) * _appSpeed,
		         (_v select 2)];
		sleep 0.1;
		_plane setDir ([_plane, _turnIn] call BIS_fnc_dirTo);
		sleep 0.1;
		systemChat format ["Distance to turnin: %1", _plane distance _turnIn];
	};
	systemChat "Turning in";
	while {_plane distance _approach > 100} do {
		// turn right, small uplift vector
		[_plane, 1, -45] call BIS_fnc_setPitchBank;
		_v = [getPos _plane, getPos _approach] call BIS_fnc_vectorFromXToY;
		_v = [ (_v select 0) * _appSpeed,
		         (_v select 1) * _appSpeed,
		         (_v select 2) * _appSpeed];

		sleep 0.1;
	};
	systemChat "Close to _approach";
};

_turnAndAlignOld = {
	params ["_plane", "_approach", "_nim"];

	_aligned = false;
	_appSpeed = 400 / 3.6;
	_turn = "none";

	while { !_aligned && alive _plane } do {
		_dir = [_plane, getPos _approach] call BIS_fnc_relativeDirTo;
		if (_dir > 180) then {
			systemChat "Left turn";
			_dir = getDir _plane + 5;
			[_plane, 1, -45] call BIS_fnc_setPitchBank;
			_turn = "left";
			// left turn
		} else {
			systemChat "Right turn";
			// right turn
			_dir = getDir _plane - 5;
			[_plane, 1, 45] call BIS_fnc_setPitchBank;
			_turn = "right";
		};
		systemChat format ["_dir is %1", _dir];
		if ( (_turn == "right" and _dir > 180) or
			 (_turn == "left" and _dir < 0) ) then {
			_aligned = true;
		} else {
			_dir = getDir _plane;
			_pos = getPos _plane;
			_pos = CIRCLEPOS2(_pos, _dir, 50);
			_pos pushBack (getPosASL _plane select 2);
			_v = [getPos _plane, _pos] call BIS_fnc_vectorFromXToY;
			_v = [ (_v select 0) * _appSpeed,
			         (_v select 1) * _appSpeed,
			         (_v select 2) * _appSpeed];

		};
		sleep 0.05;
	};

	systemChat "Finished turn to approach";
	_plane setDir ([_plane, getPos _approach] call BIS_fnc_dirTo);
	// Too much AI interference
	{ _x setDamage 1 } forEach crew _plane;

	_appSpeed = 400 / 3.6;
	while {_plane distance _approach > 50 && alive _plane} do {
		[_plane, 10, 0] call BIS_fnc_setPitchBank;
		_dir = ([_plane, getPos _approach] call BIS_fnc_relativeDirTo);
		systemChat format ["dist is %1, _dir is %2 ", _plane distance _approach, _dir];
		if (_dir != 0) then {
			_plane setDir ([_plane, getPos _approach] call BIS_fnc_dirTo);
		};

		_pos = getPos _approach;
		_v = [getPos _plane, _pos] call BIS_fnc_vectorFromXToY;
		systemChat format ["Flying to approach: %1", _v];
		_v = [ (_v select 0) * _appSpeed,
		         (_v select 1) * _appSpeed,
		         (_v select 2) * _appSpeed];
		sleep 0.05;
	};

	systemChat "Finished again, at approach";
	{ deleteVehicle _x } forEach crew _plane;
	createVehicleCrew _plane;

};

// Creates a tractor at specified starting position
_createTractor = {
	private [ "_dir", "_tractor", "_soldier", "_driverType" ];
	params ["_nim", "_startPos"];
	_tractor = _nim getVariable ["TTT_flightdeckTractor", ObjNull];
	if (isNull _tractor) then {
		_soldiers = [] call TTT_fnc_soldierTypes;
		_driverType = _soldiers select YELLOW;

		_dir = getDir _nim;

		_tractor = ([] call TTT_fnc_tractorTypes) createVehicle _startPos;
		_startPos = [_startPos select 0, _startPos select 1, DECK];
		_tractor setPosASL _startPos;
		_tractor setVelocity [0,0,-1];
		_tractor setDir _dir;

		_grp = createGroup west;
		_driverType createUnit [[0,0,0], _grp];
		(units _grp select 0) moveInDriver _tractor;
		_nim setVariable ["TTT_flightdeckTractor", _tractor, true];
	};

	// wait for tractor to drop on deck
	sleep 1;
	_tractor engineOn true;
	_tractor;
};

_attachTo = {
	params ["_tractor", "_plane"];
	[_tractor] call LESH_towing_fnc_checkTowable;
	[_tractor, _plane] call LESH_towing_fnc_tow;
	_tractor;
};

_detach = {
	params ["_tractor"];
	[_tractor] call LESH_towing_fnc_detach;
	_tractor;
};

_driveTo = {
    private ["_v"];
	params [ "_tractor", "_dest"];
	while {_tractor distance _dest > 1} do {
	    _v = [getPosASL _tractor, _dest] call BIS_fnc_vectorFromXToY;
	    _tractor setVelocity [
		  (_v select 0) * SPEEDTAXI,
		  (_v select 1) * SPEEDTAXI,
		  0
	    ];
	    sleep 0.05;
	};

};

_towToCatAndLaunch = {
	private ["_pos", "_dir", "_tractor"];
	params [ "_plane", "_posCat", "_posWires", "_parking", "_nim" ];

	_tractor = [_nim, _parking] call _createTractor;
	_pos = getPos _plane;
	_dir = getDir _plane;
	_pos = CIRCLEPOS2(_pos, _dir, 11);
	_pos pushBack DECK;
	[_tractor, _pos] call _driveTo;
	sleep 1;
	_tractor setDir _dir;
	sleep 1;
	_tractor = [_tractor, _plane] call _attachTo;

	_dir = [_tractor, _posWires] call BIS_fnc_dirTo;
	_tractor setDir _dir;
	[_tractor, _posWires] call _driveTo;

	_dir = getDir _nim;
	_pos = CIRCLEPOS2(_posCat, _dir, 10);
	_pos pushBack DECK;
	_dir = [_tractor, _pos] call BIS_fnc_dirTo;
	_tractor setDir _dir;
	[_tractor, _pos] call _driveTo;
	_tractor = [_tractor] call _detach;
	sleep 0.1;
	_plane setVelocity [0,0,0];
	[_plane, _nim, _tractor] spawn {
		params ["_plane", "_nim", "_tractor"];
		waitUntil {_plane distance _tractor > 10};
		_plane animate ["l_wingfold",0];
		_plane animate ["r_wingfold",0];
		waitUntil {_plane distance _tractor > 25};
		[_nim, _plane, 3] call TTT_fnc_planeStart;
	};
	_dir = getDir _nim + 90;
	_pos = CIRCLEPOS2(_pos, _dir, 20);
	_pos pushBack DECK;
	_dir = [_tractor, _pos] call BIS_fnc_dirTo;
	_tractor setDir _dir;
	[_tractor, _pos] call _driveTo;
	sleep 1;
	_dir = [_tractor, _parking] call BIS_fnc_dirTo;
	_tractor setDir _dir;
	[_tractor, _parking] call _driveTo;
	_tractor engineOn false;
	_tractor;
};

_towToPos = {
	private ["_tractor", "_dir", "_pos"];
	params ["_plane", "_positions", "_nim", "_parking"];

	if (!alive _plane) exitWith { ObjNull; };

	_dir = [_plane, _positions select 0] call BIS_fnc_dirTo;
	_plane setDir _dir;
	_plane animate ["l_wingfold", 1];
	_plane animate ["r_wingfold", 1];

	{ deleteVehicle _x; } forEach crew _plane;

	_tractor = [_nim, _parking] call _createTractor;
	_pos = getPos _plane;
	_pos = CIRCLEPOS2(_pos, _dir, 11);
	_pos pushBack DECK;
	_tractor setDir ([_tractor, _pos] call BIS_fnc_dirTo);
	[_tractor, _pos] call _driveTo;
	sleep 1;
	_tractor setDir (getDir _plane);
	[_tractor, _plane] call _attachTo;
	{
		_dir = [_tractor, _x] call BIS_fnc_dirTo;
		_tractor setDir _dir;
		[_tractor, _x] call _driveTo;
		sleep 1;
	} forEach _positions;
	_tractor;
};

_pushOnEle = {
    private ["_dir"];
	params ["_tractor", "_pos"];
	_dir = [_tractor, _pos] call BIS_fnc_dirTo;
	_tractor setDir (_dir - 180);
	[_tractor, _pos] call _driveTo;
	sleep 1;
	[_tractor] call LESH_towing_fnc_detach;
	_tractor;
};

_putPlaneInHangar = {
 	params ["_eleObj", "_plane", "_delay"];
 	// need to have tractor drive away from elevator
 	sleep _delay;

	[_eleObj, 1] call TTT_fnc_elevator;
	waitUntil {_eleObj animationPhase "ani_ElevBarB" < -0.9};

	// just delete the plane for now
	{ deleteVehicle _x } forEach crew _plane;
	deleteVehicle _plane;

 	[_eleObj, 0] call TTT_fnc_elevator;
};

// Prepare the flight operations, launching tanker and rescue helicopter
_prepareRescueOps = {
	private ["_helo", "_wp", "_pos", "_dir", "_grp"];
	params ["_nim"];
	_helo = H;

	// set up rescue helo to fly circles port of the carrier
	createVehicleCrew _helo;
	_grp = group driver _helo;
	_helo engineOn true;
	_pos = getPos _nim;
	_dir = getDir _nim - 90;
	_pos = CIRCLEPOS2(_pos, _dir, 500);
	_pos pushBack 100;
	_wp = _grp addWaypoint [_pos, 0];
	_wp setWaypointSpeed "SLOW";
	_wp setWaypointType "LOITER";
	/*_pos = getPos _nim;*/
/*	_dir = _dir - 90;
	_pos = CIRCLEPOS2(_pos, _dir, 500);
	_pos pushBack 100;
	_wp = _grp addWaypoint [_pos, 0];
	_wp setWaypointSpeed "SLOW";
	_wp setWaypointType "CYCLE";
*/	// what about low on fuel?
	[_helo] spawn {
		params ["_helo"];
		_helo setFuel 1;
		sleep 1800;
	};

};

_pos = getPos _nim;
_takeOffPos = CIRCLEPOS2(_pos, _dir + 45, 1000);
TakeOff = "Land_HelipadEmpty_F" createVehicle _takeOffPos;
TakeOff setPosASL [_takeOffPos select 0, _takeOffPos select 1, 200];

// create tractor
_tractor = [_nim, _parkingTractor] call _createTractor;

// setup rescue helo
[_nim] call _prepareRescueOps;

// set up tankers starboard
_pos = getPos _nim;
_dir = getDir _nim + 90;
_posTankerLoop1 = CIRCLEPOS2(_pos, _dir, 2000);
_posTankerLoop1 pushBack TANKERALT;
_posTankerLoop2 = CIRCLEPOS2(_pos, _dir, 10000);
_posTankerLoop2 pushBack TANKERALT;

/*[T1, _posCat, _posWires, _parkingTractor, _nim] call _towToCatAndLaunch;
_grp = group driver T1;
_wp = _grp addWaypoint[_posTankerLoop1, 500];
_wp = _grp addWaypoint[_posTankerLoop2, 500];
_wp setWaypointType "CYCLE";
[T2, _posCat, _posWires, _parkingTractor, _nim] call _towToCatAndLaunch;
_grp = group driver T2;
_wp = _grp addWaypoint[_posTankerLoop1, 500];
_wp = _grp addWaypoint[_posTankerLoop2, 500];
_wp setWaypointType "CYCLE";
*/
while {_doFlightOps} do {
	// spawn planes and send them around the Nimitz and recover them eventually

	[_ele3Obj, 1] call TTT_fnc_elevator;
	waitUntil {_ele3Obj animationPhase "ani_ElevBarB" < -0.9};
	_dir = getDir _nim;
	_pos = CIRCLEPOS2(_posE3, _dir - 90, -1);
    _plane = [_jetOne, _pos, HANGAR, _dir - 90, true] call TTT_fnc_placeVehicle;
    [_plane, "CAP"] call TTT_fnc_loadout;

	[_ele3Obj, 0] call TTT_fnc_elevator;
	waitUntil {_ele3Obj animationPhase "ani_ElevBarT" < 0.05};

	[_plane, _posCat, _posWires, _parkingTractor, _nim] call _towToCatAndLaunch;

	sleep DELAY;

	// get rid of all waypoints
	_grp = group driver _plane;
	while { (count (waypoints _grp)) > 0} do {
		deleteWaypoint ((waypoints _grp) select 0);
	};

	[_plane, _nim] spawn {
		params ["_plane", "_nim"];
		while {alive _plane} do {
			systemChat format ["Dist: %1, alt %2", floor(_plane distance _nim), floor(getPosASL _plane select 2)];
			sleep 1;
		};
	};

	_approachPos = CIRCLEPOS2(_pos, _dir - 187, 5000);
	_approach = "Land_HelipadEmpty_F" createVehicle _approachPos;
	_approach setPosASL [_approachPos select 0, _approachPos select 1, FLIGHTALT];

	_plane doMove (getPos _approach);
	_plane flyInHeight FLIGHTALT;
	waitUntil {_plane distance _approach < 1000 and alive _plane};

	if (alive _plane) then {
		_script = [_plane, _nim] execVM "fn_carrierApproach.sqf";
		waitUntil { scriptDone _script };
	};

	if (alive _plane) then {
		[_plane] call TTT_fnc_touchdown;
	};

	_tractor = [_plane, [_posEle2Approach, _posE2, _posEle2Tow1, _posEle2Tow2], _nim, _parkingTractor] call _towToPos;
	if (!isNull _tractor) then {
		[_tractor, _posEle2PushBack] call _pushOnEle;
		[_ele2Obj, _plane, 5] spawn _putPlaneInHangar;
		_dir = [_tractor, _parkingTractor] call BIS_fnc_dirTo;
		_tractor setDir _dir;
		[_tractor, _parkingTractor] call _driveTo;
		_tractor engineOn false;
	};
	sleep DELAY;
	// freshen local variable, in case flight operations have been stopped
	_doFlightOps = ["doFlightOps", _nim] call TTT_fnc_global;
};

systemChat "Flight operations stopped";